<?php

namespace Drupal\azure_smtp_oauth\Plugin\Mail;

require 'autoload.php';

use Drupal\azure_smtp_oauth\Service\RefreshTokenService;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Mail\MailFormatHelper;
use Drupal\Core\Mail\MailInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use PHPMailer\PHPMailer\Exception;
// Import PHPMailer classes into the global namespace.
use PHPMailer\PHPMailer\OAuth;
use PHPMailer\PHPMailer\PHPMailer;
use Symfony\Component\DependencyInjection\ContainerInterface;
use TheNetworg\OAuth2\Client\Provider\Azure;

/**
 * Modify the drupal mail system to use smtp when sending emails.
 *
 * Include the option to choose between plain text or HTML.
 *
 * @Mail(
 *   id = "SMTPMailSystem",
 *   label = @Translation("SMTP Mailer"),
 *   description = @Translation("Sends the message, using SMTP.")
 * )
 */
class SMTPMailSystem implements MailInterface, ContainerFactoryPluginInterface {
  /**
   * The allowHTML variable.
   *
   * @var allowHtml
   */
  protected $allowHtml;

  /**
   * The SMTP Config.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $smtpConfig;

  /**
   * Logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $logger;

  /**
   * Messenger.
   *
   * @var \Drupal\Core\Messenger\Messenger
   */
  protected $messenger;

  /**
   * Refresh Token generation.
   *
   * @var \Drupal\azure_smtp_oauth\Service\RefreshTokenService
   */
  protected $generateRefreshToken;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a SMPTMailSystem object.
   *
   * @param array $configuration
   *   The configuration array.
   * @param string $plugin_id
   *   The plug-in ID.
   * @param mixed $plugin_definition
   *   The plug-in definition.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger
   *   The logger object.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   The messenger object.
   * @param \Drupal\azure_smtp_oauth\Service\RefreshTokenService $generateRefreshToken
   *   The token object.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LoggerChannelFactoryInterface $logger, Messenger $messenger, RefreshTokenService $generateRefreshToken, ConfigFactoryInterface $config_factory) {
    $this->smtpConfig = \Drupal::config('azure_smtp_oauth.settings');
    $this->logger = $logger->get('azure_smtp_oauth');
    $this->messenger = $messenger;
    $this->generateRefreshToken = $generateRefreshToken;
    $this->configFactory = $config_factory;

  }

  /**
   * Creates an instance of the plugin.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container to pull out services used in the plugin.
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   *
   * @return static
   *   Returns an instance of this plugin.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.factory'),
      $container->get('messenger'),
      $container->get('azure_smtp_oauth.refresh_token_service'),
      $container->get('config.factory'),

    );
  }

  /**
   * Concatenate and wrap the e-mail body for either plain-text or HTML emails.
   *
   * @param array $message
   *   A message array, as described in hook_mail_alter().
   *
   * @return array
   *   The formatted $message.
   */
  public function format(array $message) {
    $this->allowHtml = $this->smtpConfig->get('smtp_allowhtml');

    // Join the body array into one string.
    $message['body'] = implode("\n\n", $message['body']);
    if ($this->allowHtml == 0) {
      // Convert any HTML to plain-text.
      $message['body'] = MailFormatHelper::htmlToText($message['body']);
      // Wrap the mail body for sending.
      $message['body'] = MailFormatHelper::wrapMail($message['body']);
    }

    return $message;
  }

  /**
   * Send the e-mail message.
   *
   * @param array $message
   *   A message array, as described in hook_mail_alter().
   *
   * @return mixed
   *   TRUE if the mail was successfully accepted, otherwise FALSE.
   *
   * @see drupal_mail()
   */
  public function mail(array $message) {
    $config = $this->configFactory->get('azure_smtp_oauth.settings');
    try {
      if (!empty($config->get('oauth_client_id'))) {
        $clientId = trim($config->get('oauth_client_id'));
      }
      if (!empty($config->get('oauth_client_secret'))) {
        $clientSecret = trim($config->get('oauth_client_secret'));
      }
      if (!empty($config->get('oauth_tenant_id'))) {
        $tenantId = trim($config->get('oauth_tenant_id'));
      }
    }
    catch (\Exception $e) {
      $this->logger->error('Error retrieving configuration values: ' . $e->getMessage());
      return 'Error while retrieving configuration values';
    }

    // Create a new PHPMailer instance.
    $mail = new PHPMailer(TRUE);

    // Tell PHPMailer to use SMTP.
    try {
      $mail->IsSMTP();
      $mail->Host = $this->smtpConfig->get('smtp_host');
      $mail->Port = $this->smtpConfig->get('smtp_port');
      $mail->SMTPAuth = TRUE;
      $mail->AuthType = 'XOAUTH2';
      $emailfrom = $this->smtpConfig->get('smtp_from');
      $username = $this->smtpConfig->get('smtp_fromname');
      $mail->SMTPDebug = 3;
      $mail->isHTML(TRUE);
      $sendmailto = $this->smtpConfig->get('smtp_test_address');
      $mail->addAddress($sendmailto, $username);
      $mail->Subject = 'Drupal SMTP test e-mail with MS Exchange credentials';
      $mail->Body = 'If you receive this message it means your site is capable of using SMTP to send e-mail using MS Exchange Credentials.';

      $provider = new Azure([
        'clientId' => $clientId,
        'clientSecret' => $clientSecret,
        'tenantId' => $tenantId,
      ]);
      $provider->scope = 'offline_access https://outlook.office.com/SMTP.Send';

      if (!empty($config->get('oauth_refresh_token'))) {
        $oauthRefreshToken = trim($config->get('oauth_refresh_token'));
        $refreshToken = $this->generateRefreshToken->getRefreshToken($oauthRefreshToken);
        // Pass the OAuth provider instance to PHPMailer.
        $mail->setOAuth(
        new OAuth(
          [
            'provider' => $provider,
            'clientId' => $clientId,
            'clientSecret' => $clientSecret,
            'refreshToken' => $refreshToken,
            'userName' => $emailfrom,
          ]));
        $this->logger->notice('Generated token is: ' . $refreshToken);
      }
      else {
        $this->logger->error('Could not send mail as Refresh Token is not available.');
        return FALSE;
      }
      $mail->send();
      $this->messenger->addMessage('Email sent successfully.');
    }
    catch (\Exception $e) {
      $this->logger->error('Error while sending email %e', ['%e' => $e->getMessage()]);
      return FALSE;
    }
    catch (Exception $e) {
      $this->logger->error('SMTP Error: %message', ['%message' => $e->getMessage()]);
      return FALSE;
    }
  }

}
